﻿using System;
using System.Diagnostics;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using Common;
using Common.Commands;

namespace Logic.Communications
{

    public class Communicator
    {

        private readonly Reciever _reciever;

        private TcpClient _clientSocket;
        private readonly string _envisaLinkdTpiServer;
        private const int TpiPort = 4025;
        private Thread _listenThread;
        private Thread _pollThread;
        private readonly EventLog _eventLog;
        public Communicator(string envisaLinkdTpiServer, EventLog eventLog)
        {
            _eventLog = eventLog; 
            _envisaLinkdTpiServer = envisaLinkdTpiServer;
            _clientSocket = new TcpClient();
            _reciever = new Reciever(_clientSocket, _eventLog);
        }

        public void Connect()
        {
            _clientSocket.Connect(_envisaLinkdTpiServer, TpiPort);
            _listenThread = new Thread(ListenForData);
            _listenThread.Start(); // start listening thread.
        }

        private void ReConnect()
        {
            //wait 1 second.
            Thread.Sleep(1000);
            _clientSocket = new TcpClient(); // new one
            _reciever.ReplaceClient(_clientSocket);
            _clientSocket.Connect(_envisaLinkdTpiServer, TpiPort);
            _listenThread = new Thread(ListenForData);
            _listenThread.Start(); // start listening thread.
        }

        public void EnablePoll()
        {
            if (_pollThread == null || !_pollThread.IsAlive)
            { // only start if it aint started.
                _pollThread = new Thread(Poll);
                _pollThread.Start(); // start listening thread.
            }
        }

        public void ExecuteRequest(TpiRequest tpiRequest)
        {
            var data = Encoding.ASCII.GetBytes(tpiRequest.Data);
            var serverStream = _clientSocket.GetStream();
            serverStream.Write(data, 0, data.Length);
        }

        public void Subscribe(IResponseHandler responseEventHandler)
        {
            _reciever.Subscribe(responseEventHandler);
        }
        public void UnSubscribe(IResponseHandler responseEventHandler)
        {
            _reciever.UnSubscribe(responseEventHandler);
        }

        private void Log(string message)
        {
            if (_eventLog != null)
            {
                _eventLog.WriteEntry(message);
            } 
        }


        private void ListenForData()
        {
            try
            {
                _reciever.Run();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                //nothing, check shutdown flag.
            }

            if (!_reciever.Shutdown) // if not in shutdown mode, try to restart
            {
                ReConnect(); // restart
            }

        }
        private void Poll()
        {
            var r = new TpiRequest(RequestCommand.Poll);
            while (!_reciever.Shutdown)
            {
                ExecuteRequest(r);
                try
                {
                    Thread.Sleep(60 * 1000 * 2); // sleep 2 minutes before next poll
                }
                catch
                {
                    //nothing, check shutdown flag.
                }
            }
        }

        public void Quit()
        {
            _reciever.Shutdown = true;
            if (_pollThread != null)
            {
                _pollThread.Interrupt();
            }
            if (_listenThread != null)
            {
                _listenThread.Interrupt();
            }
        }
    }
}
