﻿using System;
using System.Configuration;
using Common;
using Common.Commands;
using Plugins.mail;

namespace Plugins
{
    public class PowerHandler : AbstractResponseEventHandler
    {
        private DateTime _lostPowerTime;
        private readonly MailSender _mailSender;
        public PowerHandler()
        {
            //check if we have server and password.
            try
            {
                var v = new AppSettingsReader();
                var smtpserver = v.GetValue("smtpserver", typeof(string)) as string;
                var to = v.GetValue("to", typeof(string)) as string;
                var from = v.GetValue("from", typeof(string)) as string;
                if (smtpserver != null && to != null && from != null)
                {
                    _mailSender = new MailSender(smtpserver, from, to);
                }
            }
            catch (Exception e)
            {
                //error reading mailconfig.
            }

        }

        public override void EventOccured(IResponse response, IReciever r)
        {
            if (response.Command == ResponseCommand.PanelACTrouble)
            {
                _mailSender.SendMail("Alarm - Power", response.Date + "\r\nLost AC Power");
                _lostPowerTime = response.Date;
            }
            else if (response.Command == ResponseCommand.PanelACRestore)
            {
                var duration = "unknown";
                if (response.Date.Ticks > _lostPowerTime.Ticks)
                {
                    var span = new TimeSpan(response.Date.Ticks - _lostPowerTime.Ticks);
                    var minutes = (int)(span.TotalSeconds / 60);
                    var seconds = (int)(span.TotalSeconds - minutes * 60);
                    duration = minutes + ":" + seconds + "(min:sec)";

                }
                _mailSender.SendMail("Alarm - Power", response.Date + "\r\nAC Power Restored\nDuration: " + duration);
            }
        }
        public string GetName()
        {
            return "PowerHandler";
        }
    }
}
