﻿using System;
using System.Configuration;
using System.Diagnostics;
using Common;
using Common.Commands;
using Plugins.mail;

namespace Plugins
{
    public class ZoneHandler : AbstractResponseEventHandler
    {
        private bool AwayArmed { get; set; }
        private bool StayArmed { get; set; }

        private const string LogSource = "Alarm";
        private const string LogName = "AlarmLog";


        private readonly MailSender _mailSender;
        public ZoneHandler()
        {
            if (!EventLog.SourceExists("Alarm"))
            {
                EventLog.CreateEventSource(LogSource, LogName);
            }

            //check if we have server and password.
            try
            {
                var v = new AppSettingsReader();
                var smtpserver = v.GetValue("smtpserver", typeof(string)) as string;
                var to = v.GetValue("to", typeof(string)) as string;
                var from = v.GetValue("from", typeof(string)) as string;
                if (smtpserver != null && to != null && from != null)
                {
                    _mailSender = new MailSender(smtpserver, from, to);
                    EventLog.WriteEntry(LogSource, "Inited MailSender with server: " + smtpserver + ", to: " + to + ", from: " + from);

                }
            }
            catch (Exception e)
            {
                //error reading mailconfig.
                _mailSender = new MailSender();
                EventLog.WriteEntry(LogSource, e.Message);
            }

        }

        public override void EventOccured(IResponse response, IReciever r)
        {
            if (response.Command == ResponseCommand.ZoneAlarm)
            {
                _mailSender.SendMail("Alarm - ZoneAlarm", response.Date + "\r\nAlarm at zone: " + response.Data.Substring(1));

            }
            else if (response.Command == ResponseCommand.ZoneTamper)
            {
                _mailSender.SendMail("Alarm - ZoneTamper", response.Date + "\r\nTamper at zone: " + response.Data.Substring(1));
            }
            else if (response.Command == ResponseCommand.ZoneOpen)
            {
                //not doing anything
            }
            else if (response.Command == ResponseCommand.ZoneFault)
            {
                _mailSender.SendMail("Alarm - ZoneFault", response.Date + "\r\nFault at zone: " + response.Data.Substring(1));
            }
            else if (response.Command == ResponseCommand.PartitionArmed)
            {
                if (response.Data.Substring(1).Equals("0"))
                {
                    _mailSender.SendMail("Alarm - Armed status", response.Date + "\r\nPartition armed (away mode): " + response.Data.Substring(1));
                    AwayArmed = true;
                }
                if (response.Data.Substring(1).Equals("1"))
                {
                    _mailSender.SendMail("Alarm - Armed status", response.Date + "\r\nPartition armed (stay mode): " + response.Data.Substring(1));
                    StayArmed = true;
                }

            }
            else if (response.Command == ResponseCommand.PartitionDisarmed)
            {
                _mailSender.SendMail("Alarm - Armed status", response.Date + "\r\nPartition disarmed: " + response.Data.Substring(1));
                StayArmed = false;
                AwayArmed = false;
            }

        }

    }
}
